﻿using System;
using System.Linq;
using HeuristicLab.Analysis.FitnessLandscape.DataTables;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.FitnessLandscape.Algorithms {

  [Item("RepeatedUpDownWalkAnalyzer", "Analyzes and consolidates repeated up down walk analyses.")]
  [StorableClass]
  public class RepeatedUpDownWalkAnalyzer : SingleSuccessorOperator, IRepeatsAnalyzer {

    #region Parameters
    public ScopeTreeLookupParameter<DoubleValue> DownWalkLengthValuesParameter {
      get { return (ScopeTreeLookupParameter<DoubleValue>)Parameters["DownWalkLengthValues"]; }
    }
    public ScopeTreeLookupParameter<DoubleValue> UpWalkLengthValuesParameter {
      get { return (ScopeTreeLookupParameter<DoubleValue>)Parameters["UpWalkLengthValues"]; }
    }
    public ScopeTreeLookupParameter<DoubleValue> DownWalkLenVarValuesParameter {
      get { return (ScopeTreeLookupParameter<DoubleValue>)Parameters["DownWalkLenVarValues"]; }
    }
    public ScopeTreeLookupParameter<DoubleValue> UpWalkLenVarValuesParameter {
      get { return (ScopeTreeLookupParameter<DoubleValue>)Parameters["UpWalkLenVarValues"]; }
    }
    public LookupParameter<DoubleValue> DownWalkLengthParameter {
      get { return (LookupParameter<DoubleValue>)Parameters["DownWalkLength"]; }
    }
    public LookupParameter<DoubleValue> UpWalkLengthParameter {
      get { return (LookupParameter<DoubleValue>)Parameters["UpWalkLength"]; }
    }
    public LookupParameter<DoubleValue> DownWalkLenVarParameter {
      get { return (LookupParameter<DoubleValue>)Parameters["DownWalkLenVar"]; }
    }
    public LookupParameter<DoubleValue> UpWalkLenVarParameter {
      get { return (LookupParameter<DoubleValue>)Parameters["UpWalkLenVar"]; }
    }
    public LookupParameter<ResultCollection> ResultsParameter {
      get { return (LookupParameter<ResultCollection>)Parameters["Results"]; }
    }
    #endregion

    #region Construction & Cloning
    [StorableConstructor]
    protected RepeatedUpDownWalkAnalyzer(bool deserializing) : base(deserializing) { }
    protected RepeatedUpDownWalkAnalyzer(RepeatedUpDownWalkAnalyzer original, Cloner cloner) : base(original, cloner) { }
    public RepeatedUpDownWalkAnalyzer() {
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("DownWalkLengthValues", "average down length values of repeats.", "DownWalkLength", 2));
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("UpWalkLengthValues", "average up walk length values of repeats.", "UpWalkLength", 2));
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("DownWalkLenVarValues", "down length variance values of repeats.", "DownWalkLenVar", 2));
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("UpWalkLenVarValues", "down length variance values of repeats.", "UpWalkLenVar", 2));

      Parameters.Add(new LookupParameter<DoubleValue>("DownWalkLength", "Consolidated average down walk length."));
      Parameters.Add(new LookupParameter<DoubleValue>("UpWalkLength", "Consolidated average up walk length."));
      Parameters.Add(new LookupParameter<DoubleValue>("DownWalkLenVar", "Consolidated down walk length variance."));
      Parameters.Add(new LookupParameter<DoubleValue>("UpWalkLenVar", "Consolidated up walk length variance."));

      Parameters.Add(new LookupParameter<ResultCollection>("Results", "The collection of all results."));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new RepeatedUpDownWalkAnalyzer(this, cloner);
    }
    #endregion

    public override IOperation Apply() {
      AggregateValues(DownWalkLengthValuesParameter, DownWalkLengthParameter);
      AggregateValues(UpWalkLengthValuesParameter, UpWalkLengthParameter);
      AggregateValues(DownWalkLenVarValuesParameter, DownWalkLenVarParameter);
      AggregateValues(UpWalkLenVarValuesParameter, UpWalkLenVarParameter);
      return base.Apply();
    }

    private void AggregateValues(ILookupParameter<ItemArray<DoubleValue>> valuesParameter, ILookupParameter<DoubleValue> averageParameter) {
      var results = ResultsParameter.ActualValue;
      var values = valuesParameter.ActualValue.Select(v => v.Value).ToList();
      if (values.Count > 0) {
        var avg = new DoubleValue(values.Average());
        averageParameter.ActualValue = avg;
        results.Remove(averageParameter.ActualName);
        results.Add(new Result(averageParameter.ActualName, avg));
      }
    }

  }
}
